Package: SharedObject
Type: Package
Title: Sharing R objects across multiple R processes without memory
        duplication
Version: 1.26.2
Date: 2023-8-9
Authors@R: c(
    person(
        "Jiefei", "Wang",
        email = "szwjf08@gmail.com",
        role = c("aut", "cre")
    ),
    person("Martin", "Morgan", role = "aut"))
Description: This package is developed for facilitating parallel
        computing in R. It is capable to create an R object in the
        shared memory space and share the data across multiple R
        processes. It avoids the overhead of memory dulplication and
        data transfer, which make sharing big data object across many
        clusters possible.
License: GPL-3
LinkingTo: BH, Rcpp
Depends: R (>= 3.6.0)
Imports: Rcpp, methods, stats, BiocGenerics
biocViews: Infrastructure
BugReports: https://github.com/Jiefei-Wang/SharedObject/issues
Suggests: testthat, parallel, knitr, rmarkdown, BiocStyle
RoxygenNote: 7.1.1
Roxygen: list(markdown = TRUE)
VignetteBuilder: knitr
SystemRequirements: GNU make, C++11
Encoding: UTF-8
Config/pak/sysreqs: make
Repository: Bioconductor 3.23
Date/Publication: 2026-08-12 14:55:28 UTC
RemoteUrl: https://github.com/bioc/SharedObject
RemoteRef: RELEASE_3_23
RemoteSha: 62f14b1fce30499112be8b9040021368748b765f
NeedsCompilation: yes
Packaged: 2026-08-12 21:35:52 UTC; root
Author: Jiefei Wang [aut, cre],
  Martin Morgan [aut]
Maintainer: Jiefei Wang <szwjf08@gmail.com>
Built: R 4.6.1; aarch64-apple-darwin23; 2026-08-12 21:37:43 UTC; unix
