# Generated by roxygen2: do not edit by hand

export(SharedObject)
export(allocateSharedMemory)
export(freeSharedMemory)
export(getCopyOnWrite)
export(getLastIndex)
export(getSharedCopy)
export(getSharedMemorySize)
export(getSharedSubset)
export(hasSharedMemory)
export(initialSharedObjectPackageData)
export(is.shared)
export(listSharedObjects)
export(mapSharedMemory)
export(releaseSharedObjectPackageData)
export(setCopyOnWrite)
export(setSharedCopy)
export(setSharedSubset)
export(share)
export(sharedObjectPkgOptions)
export(sharedObjectProperties)
export(unmapSharedMemory)
export(unshare)
exportMethods(is.shared)
exportMethods(share)
exportMethods(sharedObjectProperties)
exportMethods(unshare)
import(BiocGenerics)
import(methods)
importFrom(Rcpp,sourceCpp)
importFrom(methods,as)
importFrom(methods,is)
importFrom(methods,new)
importFrom(stats,runif)
useDynLib(SharedObject, .registration = TRUE)
