# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

C_getDataInfoTemplate <- function() {
    .Call(`_SharedObject_C_getDataInfoTemplate`)
}

C_getStringDataInfoTemplate <- function() {
    .Call(`_SharedObject_C_getStringDataInfoTemplate`)
}

C_createEmptySharedObject <- function(type, length, copyOnWrite, sharedSubset, sharedCopy, attributes) {
    .Call(`_SharedObject_C_createEmptySharedObject`, type, length, copyOnWrite, sharedSubset, sharedCopy, attributes)
}

C_createSharedObjectFromSource <- function(x, copyOnWrite, sharedSubset, sharedCopy, attributes) {
    .Call(`_SharedObject_C_createSharedObjectFromSource`, x, copyOnWrite, sharedSubset, sharedCopy, attributes)
}

C_createSharedStringFromSource <- function(x, copyOnWrite, attributes) {
    .Call(`_SharedObject_C_createSharedStringFromSource`, x, copyOnWrite, attributes)
}

C_readSharedObject <- function(dataInfo) {
    .Call(`_SharedObject_C_readSharedObject`, dataInfo)
}

C_unshare <- function(x, attributes) {
    .Call(`_SharedObject_C_unshare`, x, attributes)
}

C_unshareString <- function(x, attributes) {
    .Call(`_SharedObject_C_unshareString`, x, attributes)
}

C_setSharedObjectOwership <- function(x, ownData) {
    invisible(.Call(`_SharedObject_C_setSharedObjectOwership`, x, ownData))
}

C_getSharedObjectOwership <- function(x) {
    .Call(`_SharedObject_C_getSharedObjectOwership`, x)
}

C_xlength <- function(x) {
    .Call(`_SharedObject_C_xlength`, x)
}

C_ALTREP <- function(x) {
    .Call(`_SharedObject_C_ALTREP`, x)
}

C_getAltData1 <- function(x) {
    .Call(`_SharedObject_C_getAltData1`, x)
}

C_getAltData2 <- function(x) {
    .Call(`_SharedObject_C_getAltData2`, x)
}

C_isShared <- function(x) {
    .Call(`_SharedObject_C_isShared`, x)
}

C_setAltData1 <- function(x, data) {
    invisible(.Call(`_SharedObject_C_setAltData1`, x, data))
}

C_setAltData2 <- function(x, data) {
    invisible(.Call(`_SharedObject_C_setAltData2`, x, data))
}

C_getObject <- function(x) {
    .Call(`_SharedObject_C_getObject`, x)
}

C_setObject <- function(x, i) {
    invisible(.Call(`_SharedObject_C_setObject`, x, i))
}

C_ISS4 <- function(x) {
    .Call(`_SharedObject_C_ISS4`, x)
}

C_SETS4 <- function(x) {
    invisible(.Call(`_SharedObject_C_SETS4`, x))
}

C_UNSETS4 <- function(x) {
    invisible(.Call(`_SharedObject_C_UNSETS4`, x))
}

C_isSameObject <- function(x, y) {
    .Call(`_SharedObject_C_isSameObject`, x, y)
}

C_getDataTypeId <- function(type) {
    .Call(`_SharedObject_C_getDataTypeId`, type)
}

C_initialPkgData <- function() {
    invisible(.Call(`_SharedObject_C_initialPkgData`))
}

C_releasePkgData <- function() {
    invisible(.Call(`_SharedObject_C_releasePkgData`))
}

C_getLastIndex <- function() {
    .Call(`_SharedObject_C_getLastIndex`)
}

C_allocateSharedMemory <- function(size_in_byte, name = "") {
    .Call(`_SharedObject_C_allocateSharedMemory`, size_in_byte, name)
}

C_mapSharedMemory <- function(id) {
    .Call(`_SharedObject_C_mapSharedMemory`, id)
}

C_unmapSharedMemory <- function(id) {
    invisible(.Call(`_SharedObject_C_unmapSharedMemory`, id))
}

C_freeSharedMemory <- function(id) {
    invisible(.Call(`_SharedObject_C_freeSharedMemory`, id))
}

C_hasSharedMemory <- function(id) {
    .Call(`_SharedObject_C_hasSharedMemory`, id)
}

C_getSharedMemorySize <- function(id) {
    .Call(`_SharedObject_C_getSharedMemorySize`, id)
}

C_getSharedMemoryPath <- function() {
    .Call(`_SharedObject_C_getSharedMemoryPath`)
}

createSharedObjectFromSource <- function(x, copyOnWrite, sharedSubset, sharedCopy, attributes) {
    .Call(`_SharedObject_createSharedObjectFromSource`, x, copyOnWrite, sharedSubset, sharedCopy, attributes)
}

createSharedStringFromSource <- function(x, copyOnWrite, attributes) {
    .Call(`_SharedObject_createSharedStringFromSource`, x, copyOnWrite, attributes)
}

getSharedObjectList <- function() {
    .Call(`_SharedObject_getSharedObjectList`)
}

C_setSharedMemoryPrint <- function(x) {
    invisible(.Call(`_SharedObject_C_setSharedMemoryPrint`, x))
}

C_setAltrepPrint <- function(x) {
    invisible(.Call(`_SharedObject_C_setAltrepPrint`, x))
}

C_setPackagePrint <- function(x) {
    invisible(.Call(`_SharedObject_C_setPackagePrint`, x))
}

getSharedMemoryPath <- function() {
    .Call(`_SharedObject_getSharedMemoryPath`)
}

